#!/usr/bin/env Rscript

library(plyr)
library(dplyr)
library(tidyr)
library(dada2)
library(ggplot2)

main <- function() {
    # Recuperer les arguments passes au programme
    args = commandArgs(trailingOnly=TRUE)

    # Dossier ou se trouvent les fichiers a traiter (tables asv au format .rds (attention a n'avoir que des tables asv à merger dans ce dossier, sinon il va essayer de merge tous les .rds))
    inputpath = args[1]

    # Dossier ou on va ecrire les sorties
    outputpath = args[2]
   
    #Merging several ASV Tables merge_asv_process 
    merge_process_only = args[3]

    #select rds files in inputpath, asv tables to merge must be named SOMETHING_asv.rds
    file.names <- list.files(inputpath, pattern ="_asv.rds", full.names=TRUE)
    print(paste0("ASV tables going to be merged : ", file.names))
    mergetab <- mergeSequenceTables(tables=file.names, repeats="sum")
  
    if (merge_process_only=="no") {
        #ligation=yes, read.track files must be merged
        # bind the two read track dataframes together by row and aggregate
        track1=readRDS(paste0(inputpath, "/8_readtrackR1FR2R.rds"))
        track2=readRDS(paste0(inputpath, "/8_readtrackR2FR1R.rds"))
        #track2=read.table(file=paste0(inputpath, "/8_readtrackR2FR1R.csv"), sep=";", col.names=T, row.names=F)
        read.track.final=aggregate(. ~ sample, rbind(track1, track2),sum)
        read.track.final$final_perc_reads_retained <- round(as.numeric(as.character(read.track.final$non_chimeric_reads))/as.numeric(as.character(read.track.final$reads.in))*100, 1) 
	write.table(read.track.final, file=paste0(outputpath, "/8_readtrackfinal.csv"), sep=";", col.names=T, row.names=F)
    }

    #save final asv table
    saveRDS(mergetab, paste0(outputpath, "/9_asvmergedfinal.rds"))
}

if (!interactive()) {
        main()
}
