#!/usr/bin/env Rscrip

library(plyr)
library(dplyr)
library(tidyr)
library(dada2)
library(ggplot2)

dadaoutputfiles <- function(path, finaltab, rdpbank) {
    # Assignation taxonomique des ASV
    print("## Assigning ASV taxonomy ## ")
    taxa <- assignTaxonomy(finaltab, rdpbank, minBoot=80, verbose=TRUE, outputBootstraps=TRUE, tryRC=TRUE, multithread=TRUE)
    #saveRDS(taxa, paste0(path, "/9_taxonomy.rds"))

    # Create output data
    asv_seqs <- colnames(finaltab)
    asv_headers <- vector(dim(finaltab)[2], mode="character")
    for (i in 1:dim(finaltab)[2]) {
        asv_headers[i] <- paste(">Cluster", i, sep="_")
    }
    # ASV fasta files
    print("Write ASV fasta file")
    asv_fasta <- c(rbind(asv_headers, asv_seqs))
    write(asv_fasta, paste0(path, "/10_ASVs.fasta"))
    # Count table
    print("Write count table to file")
    asv_tab <- as.data.frame(t(finaltab))
    #rownames(asv_tab) <- sub(">", "", asv_headers)
    #asv_tab$ClusterID <- rownames(asv_tab)
    #asv_tav <- asv_tab[,c(3,1,2)]
    ClusterID <- sub(">", "", asv_headers)
    asv_tab <- cbind(ClusterID, asv_tab)
    write.table(asv_tab, paste0(path, "/10_ASVs_counts.tsv"), sep="\t", quote=F, col.names=T, row.names=F)
    # Taxa table
    print("Write taxa table to file")
    asv_tax <- as.data.frame(taxa)
    asv_tax <- cbind(ClusterID, asv_tax)
    #rownames(asv_tax) <- sub(">", "", asv_headers)
    #asv_tax$ClusterID <- rownames(asv_tax)
    #asv_tax <- asv_tax[, c(13,1,2,3,4,5,6,7,8,9,10,11,12)]
    write.table(asv_tax, paste0(path, "/10_ASVs_taxonomy.tsv"),  sep="\t", quote=F, col.names=T, row.names=F)
}

# formatforfrogs output is only needed if frogs_process is wanted, which includes swarm clustering and an assignement in FROGS via blast
formatforfrogs <- function(asvtable, outputpath) {
    # Make count table for Frogs
    # Transpose seqtab2 to have samples in columns, sequences in rows
    seqtab2T = t(asvtable)
    seqtab2Tcopy = seqtab2T
    # make cluster IDs
    id = c(1:dim(seqtab2T)[1])
    # add cluster IDs to asv table
    rownames(seqtab2T) = NULL
    seqtab2T = cbind(id,seqtab2T)
    write.table(seqtab2T, paste0(outputpath,"/1_preprocess.tsv"), sep = "\t", quote  = F, row.names = F)
    
    # Fasta for Frogs, in the format:
    # >idsequence; size=1234 1:N:0:54 
    # ACGATGACT
    # (le 1:N:0:54 n'est pas utilise, cependant mon code uclust est fait pour gerer des sequences qui contiennent ca, donc il faut l'ajouter si ce n'est pas present)
    # create fasta headers:
    lignes_id = paste0('>',as.data.frame(seqtab2T)$id,';size=',rowSums(seqtab2T[,2:(dim(seqtab2T)[2])]))
    fasta = paste0(lignes_id, "\n", rownames(seqtab2Tcopy))
    write(fasta, paste0(outputpath,"/1_preprocess.fasta"))
}

main <- function() {

    # Recuperer les arguments passes au programme
    args = commandArgs(trailingOnly=TRUE)
    
    #Path for inputs (dada_dir or output_merge) and outputs
    path = args[1]
    print(paste0("- dada_dir_path or output_merge_path : ", path))

    #ASV Table 
    finaltab = readRDS(args[2])
    print(paste0("- asv_table : ", finaltab))

    #rdpbank
    rdpbank = args[3]
    print(paste0("- RDPbank : ", rdpbank))

    if (!dir.exists(path)) {
        stop(paste0("Input folder ", path, " does not exist"))
    }
    
    dadaoutputfiles(path, finaltab, rdpbank)
    formatforfrogs(finaltab, path)
}

if (!interactive()) {
        main()
}
