# Objectif de ce script: combiner 1_preprocess.tsv avec la sortie produite par le clustering uclust pour obtenir un 2_clustering.biom et un 2_clustering.fasta reinjectables dans Frogs.

#Pour le TSV:

# 1_preprocess.tsv:
# #id		ech1	ech2
# idA		28 		2
# idB		1		5

# sortie uclust (composition des clusters):
# denovo1	idA;size=30	idC;size=6
# denovo2	idB;size=8
# denovo3	idD;size12	idE;size=1

# ce qu'on veut obtenir:
# #seed_id	observation_name	observation_sum		ech1	ech2
# idA		denovo1			30			28	2
# idB		denovo2			6			1	5	

# et il faudra transformer ca en biom.

#Pour le biom:

# Le programme attend 5 parametres: 1_preprocess.fasta, 1_preprocess.tsv, la sortie uclust, et le fasta de sortie (pas le tsv qui n'est qu'un fichier intermediaire). Plus le nombre d'echantillons, sauf si je trouve un moyen de m'en passer.

from operator import add #va nous servir pour sommer des listes d'entiers
import csv,sys,re

input_fasta = sys.argv[1]
input_tsv = sys.argv[2]
input_uclust = sys.argv[3]
output_fasta = sys.argv[4]
output_tsv = sys.argv[5]
#nbEchs = int(sys.argv[5])

def recuperer_nb_echs(in_tsv):

	with open (in_tsv,'r') as infile:
		tsvreader = csv.reader(infile, delimiter='\t')
		nbEchs = len(tsvreader.next()) - 1

        return (nbEchs) 

#Les lignes d'en-tete de 1_preprocess.tsv et celle du tsv qu'on veut generer se ressemblent violemment, puisqu'on a:
#input: #id		ech1	ech2
#output: #seed_id	observation_name	observation_sum		ech1	ech2
def copier_en_tete(input_file):

	with open (input_file,'r') as infile:
	
		tsvreader = csv.reader(infile, delimiter='\t')
		ligne_in = tsvreader.next()
		ligne_in.pop(0)
		ligne_out = ["#seed_id","observation_name","observation_sum"] + ligne_in
	
	return ligne_out

#Creer le dictionnaire de 1_preprocess.tsv: on va associer a chaque identifiant son comptage dans les differents echantillons
def associer_chaque_id_avec_son_comptage(input_tsv):

	dico = {}
	with open(input_tsv,'r') as tsv:

		tsvreader = csv.reader(tsv, delimiter='\t')
		
		#Sauter la premiere ligne qui est une ligne d'en-tete
		tsvreader.next()
	
		#Traitement des autres lignes, qui contiennent le comptage de chaque sequence sous la forme: idA 1 0 2
		for row in tsvreader:

			identifiant = row[0]
			row.pop(0)
			
			#Transformer les elements de la liste en entiers (la c'est des strings)
			row = map(int,row)
			
			#Faire l'association entre l'identifiant et ses comptages
			dico[identifiant] = row
			
	return dico
	
#Faire le comptage de chaque cluster -> sommer les comptages de toutes les sequences appartenant au cluster, puis ecrire chaque comptage dans une table de contingence
def ecrire_comptage_clusters(input_uclust,dico_comptages,dico_sequences,dico_bizarre,output_tsv,output_fasta,nbEchs):

	with open (input_uclust,'r') as uclust, open(output_tsv,'w')as tsv, open(output_fasta,'w') as out_fasta:

		tsvwriter = csv.writer(tsv, delimiter='\t')
		
		#Recuperer la ligne d'en-tete du tsv d'entree, la traiter et l'ecrire dans le tsv de sortie
		ligne_en_tete = copier_en_tete(input_tsv) #input_tsv est une variable globale
		tsvwriter.writerow(ligne_en_tete)

		#Initialisation de la variable ligne (qui contiendra successivement toutes les lignes du fichier d'entree)
		ligne = uclust.readline()
	
		#Tant qu'on n'est pas au bout du fichier
		while (ligne != ''): #une ligne uclust c'est idA;size=2	idB;size=1
	
			#Initialisation du comptage du cluster a [0,0,0,...,0]
			comptage_clusters=[0] * nbEchs
		
			ligne = ligne.split("\t") #denovo1	idA;size=2	idB;size=1  ->  ["denovo1","idA;size=2","idB;size=1"]
		
			idCluster = ligne[0] #denovo1
			ligne.pop(0)
			
			#Recuperer l'identifiant de la sequence seed:
			seed_id = ligne[0].split(";")[0] #idA;size=2 -> ["idA","size=2] -> "idA"
			
			#Initialiser la variable qui contiendra le nombre total de sequences du cluster
			observation_sum = 0
			
			#Boucler sur toutes les sequences du cluster (y compris la seed que je n'ai pas vraiment traitee)
			for sequence in ligne:
			
				sequence = sequence.split(";") #idA;size=2 -> ["idA","size=2]
				id_sequence = sequence[0]
				
				#Sommer le comptage du cluster qu'on a deja et le comptage de la sequence consideree. Ex: [0,12,3] + [4,0,6] -> [4,12,9]
				comptage_clusters = map(add,comptage_clusters,dico_comptages[id_sequence])
				
			#Obtenir le nombre total de sequences du cluster (sommer tous les echantillons)
			observation_sum = sum(comptage_clusters)				

			#Creation de la ligne qu'on va ecrire dans le fichier de sortie.
			ligne_out = [seed_id,idCluster,observation_sum] + comptage_clusters
			
			tsvwriter.writerow(ligne_out)
			out_fasta.write('>'+idCluster+' '+dico_bizarre[seed_id]+'\n')
			out_fasta.write(dico_sequences[seed_id]+'\n')
	
			#Passer a la ligne suivante
			ligne = uclust.readline()

def construire_dicos_fasta(input_fasta):
	
	dico_sequences = {}
	dico_bizarre = {}

	with open (input_fasta,'r') as in_fasta:

		ligne_id = in_fasta.readline() #Format de la ligne: >M03930:58:000000000-AWK1A:1:2107:13628:11340;size=2 1:N:0:8 
		while(ligne_id != ''):	
			ligne_id = ligne_id[1:] #Enlever le > de debut
			ligne_id = ligne_id.split(';') #["M03930:58:000000000-AWK1A:1:2107:13628:11340","size=2 1:N:0:8"]
			identifiant = ligne_id[0]
			ligne_id = ligne_id[1].split(' ') #["size=2","1:N:0:8"]
			machin_bizarre = ligne_id[1]
			sequence = in_fasta.readline()
			dico_sequences[identifiant] = sequence
			dico_bizarre[identifiant] = machin_bizarre
			ligne_id = in_fasta.readline()
	
	return ([dico_sequences,dico_bizarre])	

# MAIN

nbEchs = recuperer_nb_echs(input_tsv)
dico_comptages = associer_chaque_id_avec_son_comptage(input_tsv)
dicos_fasta = construire_dicos_fasta(input_fasta)
dico_sequences = dicos_fasta[0]
dico_bizarre = dicos_fasta[1]
ecrire_comptage_clusters(input_uclust,dico_comptages,dico_sequences,dico_bizarre,output_tsv,output_fasta,nbEchs)
